#include <windows.h>

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

#include <gl\glaux.h>		// Header File For The Glaux Library
#include <GL\glut.h>		// ogl stuff

#include "structures.h"
#include "myfuncs.h"
#include "Maths/maths.h"
#include "Julia/Julia.h"



/*** COMPILATION DIRECTIVES ***/


/*** CONSTANTS ***/

// Initial window/viewport dimensions (in pixels)
#define WIN_WIDTH	(512)
#define WIN_HEIGHT	(512)

Viewport gVP;

Frustum	gFR=
{
	-0.5*0.125f, 
	0.5*0.125f,
	-0.5*0.125f, 
	0.5*0.125f,
	0.25f,
	1000
};
enum Fmode
{
	MANDEL = 0,
	JULIA
};

Mouse	gMouse;
Julia	g_Julia;
Fmode	mode	= MANDEL;
int		g_Res	= 4;

char keys[256];	// marked 1 if down 0 if up
float theta= 0, phi= 20, Xtracking = 0, Ytracking = 0, depth= 50;


//-------------------------------------------------------------------------------------------------------------------------------
// Set basic OpenGL states
int init (void)
{	
//	srand((unsigned)time( NULL ));

	glShadeModel(GL_SMOOTH);						// Enable Smooth Shading
	glClearDepth(1.0f);							// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);						// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);							// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);

	glPolygonMode( GL_BACK, GL_FILL );					// Back Face Is Solid
	glPolygonMode( GL_FRONT, GL_LINE );					// Front Face Is Made Of Lines

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// black background

	gVP.origin.Set(0, 0);
	gVP.size.Set(WIN_WIDTH, WIN_HEIGHT);
	gMouse.drag.Set(0,0);
	gMouse.pos.Set(0,0);

	g_Julia.Calculate(1, 1);


	return TRUE;
} // init

void startFrame (void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glViewport(gVP.origin.x, gVP.origin.y, gVP.size.x, gVP.size.y);

	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();				
    glFrustum(gFR.left, gFR.right, gFR.bottom, gFR.top, gFR.zNear, gFR.zFar);	
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void endFrame (void)
{
    glutSwapBuffers();
    glutPostRedisplay();
}


void OrthoMode(int left, int top, int right, int bottom) //####################### NOT MY CODE ############
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();									
	
	glLoadIdentity();								
	glOrtho( left, right, bottom, top, 0, 1 );	
	
	glMatrixMode(GL_MODELVIEW);	
	glLoadIdentity();										
}

void PerspectiveMode()										// Set Up A Perspective View
{
	glMatrixMode( GL_PROJECTION );
	glPopMatrix();				
	glMatrixMode( GL_MODELVIEW );							

}

void input()
{	float mx,my;

	mx = gMouse.pos.x;
	my = gMouse.pos.y;


	// Modify rotation and translation according to mouse input -----------------------
	//	theta+= gMouse.drag.x;			//	phi+= gMouse.drag.y;	//	gMouse.drag.x= 0;	//	gMouse.drag.y= 0;
	//	depth -= gMouse.drag.y/1.0f;	//	gMouse.drag.y= 0;
	if (gMouse.buttons & MB_LEFT )	
	{
		g_Julia.Blank();
		if(mode == JULIA)
		{
			g_Res = 4;
			g_Julia.Zoom(mx, my, 0.5f);
		}
		else
		{
			g_Julia.SetComplexPoint(mx,my);
			g_Julia.Calculate(g_Res, g_Res);
			cout << "mouse (" << mx << "," << my << ")";
		}
	}
	if (gMouse.buttons & MB_MID)
	{
		Xtracking += gMouse.drag.x/5.5;		
		Ytracking -= gMouse.drag.y/5.0;
		gMouse.drag.x= 0;					
		gMouse.drag.y= 0;
	}

	if (gMouse.buttons & MB_RIGHT)
	{
		g_Julia.Blank();
		g_Res = 4;
		g_Julia.Zoom(mx, my, 2.0f);
	}

	if(keys['q'])
	{	g_Julia.MulItters(2);		
		g_Julia.Blank();	
		g_Res = 4;	
		keys['q'] = 0;	
	}

	if(keys['a'])
	{	
		g_Julia.MulItters(0.5f);	
		g_Julia.Blank();	
		g_Res = 4;		
		keys['a'] = 0;
	}

	if(keys['m'])
	{	
		mode = MANDEL;			
		g_Julia.Blank();	
		g_Res = 4;	
		g_Julia.SetComplexPoint(0,0);	
		g_Julia.InitZoom();	
	}

	if(keys['z']){	mode = JULIA;	}
}

void display (void)
{

	input();

	if(g_Res > 1)
	{
		g_Res--;
		g_Julia.Calculate(g_Res, g_Res);
	}

	startFrame();

	// Rotate then translate the world
    glTranslatef(Xtracking, Ytracking, -depth);
	glRotatef(theta, 0,1,0);
	glRotatef(phi, 1,0,0);


	OrthoMode(0, 0, gVP.size.x, gVP.size.y);
	g_Julia.Render();

	PerspectiveMode();
	endFrame();
}

void updateMouse (int x, int y)
{
	gMouse.drag.x= x - gMouse.pos.x;
	gMouse.drag.y= y - gMouse.pos.y;

	gMouse.pos.x= x;
	gMouse.pos.y= y;
} 

void mouse (int button, int state, int x, int y)
{
	switch (button)
	{
		case GLUT_LEFT_BUTTON :
			if (state == GLUT_DOWN)		gMouse.buttons|= MB_LEFT;
			else						gMouse.buttons&= ~MB_LEFT;
			break;

		case GLUT_MIDDLE_BUTTON :
			if (state == GLUT_DOWN)		gMouse.buttons|= MB_MID;
			else						gMouse.buttons&= ~MB_MID;
			break;

		case GLUT_RIGHT_BUTTON :
			if (state == GLUT_DOWN)		gMouse.buttons|= MB_RIGHT;
			else						gMouse.buttons&= ~MB_RIGHT;
			break;
	}
	updateMouse(x,y);
}

void motion (int x, int y)
{
	updateMouse(x,y);
}

void passive (int x, int y)
{
	updateMouse(x,y);
}

void keyboard (unsigned char key, int x, int y)
{
	int i;

	for (i=0; i<256; i++){
		if (key == i) keys[i] = 1;
		else keys[i] = 0;
	}

	updateMouse(x,y);
} 

void reshape (int x, int y)
{
	// Set viewport to size of window
		gVP.size.x= x;
		gVP.size.y= x;
		gVP.origin.x = 0;
		gVP.origin.y = 0;
} 


int main (int argc, char *argv[])
{
	// Set up basic GLUT stuff
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInitWindowSize(WIN_WIDTH, WIN_HEIGHT);
	glutInitWindowPosition(16, 16);
	glutCreateWindow("Julia");

	// Initialise program data
	init();

    // Inform GLUT of this programs IO callback functions
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutPassiveMotionFunc(passive);
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);

	glutMainLoop();

	return(0);
}