#include "Julia.h"
#include <stdio.h>
#include <GL/glut.h>

Julia::Julia() 
{ 
	m_zr = m_zi = 0;
	m_Xsize = m_Ysize = 512;
	m_Itters = 32;
	InitZoom();
}

void Julia::InitZoom()
{
	m_l = -2;
	m_r = 2;
	m_t = -2;
	m_b = 2;
	m_zoom = 4.0f;
}

void Julia::Zoom(float _x, float _y, float _amount)
{
	long double x = m_l + (((_x / 512)) * m_zoom);
	long double y = m_t + (((_y / 512)) * m_zoom);
	
	m_zoom	= m_zoom * _amount;
	m_l		= x - m_zoom * 0.5f;
	m_r		= x + m_zoom * 0.5f;
	m_t		= y - m_zoom * 0.5f;
	m_b		= y + m_zoom * 0.5f;
	
	cout << "Zoom: " << 1 / m_zoom << endl;
	cout << "Complex Point: (" << x << "," << y << ")" << endl;

}

void Julia::SetComplexPoint (float _x, float _y)
{
	long double x = m_l + (((_x / 512)) * m_zoom);
	long double y = m_t + (((_y / 512)) * m_zoom);
	
	m_zr = x;
	m_zi = y;
	
	cout << "Complex Point (" << x << "," << y << ")" << endl;
}

void Julia::Blank()
{
    for (int y = 0; y < m_Ysize; y++)
        for (int x = 0; x < m_Xsize; x++)
			buffer[x][y] = m_Itters + 1;
}

void Julia::Calculate(int _skipx, int _skipy)
{ 
	int		x, y, i;
    long double zr, zi, cr, ci, rsq, isq;	
	
    for (y = 0; y < m_Ysize; y+=_skipy)
    {
        for (x = 0; x < m_Xsize; x+=_skipx)
        {
			if(buffer[x][y] == unsigned int(m_Itters) + 1)
			{
				cr = m_zr;								//C
				ci = m_zi;
				zr = m_l + x * (m_r - m_l) / m_Xsize;	//Z0		
				zi = m_t + y * (m_b - m_t) / m_Ysize;
				rsq = zr * zr;							//Z0^2
				isq = zi * zi;
				
				for (i = 0; ((rsq + isq) <= 4.0) && (i < m_Itters); i++)
				{
					zi = zr * zi * 2;
					zi += ci;
					
					zr = rsq - isq;
					zr += cr;
					
					rsq = zr * zr;
					isq = zi * zi;
				}
				
				if (rsq + isq <= 4.0)					//Set colour of point
					buffer[x][y] = 0;   
				else
					buffer[x][y] = i;   
			}
		}
    }
}

void Julia::Render()
{
	float scale = 0.1f;
	float c;
	glColor3f(1,1,1);
	glBegin(GL_POINTS);
    for (int y = 0; y < 512; y++)
    {
        for (int x = 0; x < 512; x++)
		{
			if(buffer[x][y] > 1)
			{
				if(buffer[x][y] == unsigned int(m_Itters) + 1)
					glColor3f(0, 0, 0);
				else
				{
					c = (buffer[x][y] * (1.0f/m_Itters) );
					glColor3f(1, c, 0);
				}

				glVertex2f(x, y);				
			}
		}
	}
	glEnd();
}

void Julia::MulItters(float s)
{
	m_Itters *= s;
	cout << "Itterations:" << m_Itters << endl;
}