#include "Julia.h"
#include <stdio.h>
#include <GL/glut.h>

class imaginaryNumber
{
public:
	long double r;		//real part
	long double i;		//imaginary part

};


void Mandelbrot::Zoom(float _x, float _y, float _amount)
{
	long double x = l + (((_x / 512)) * zoom);
	long double y = t + (((_y / 512)) * zoom);

	zoom = zoom * _amount;
	l=x-zoom*0.5f;
	r=x+zoom*0.5f;
	t=y-zoom*0.5f;
	b=y+zoom*0.5f;

	Calculate(512, 512);
}

void Mandelbrot::Calculate(int _sizex, int _sizey)
{ 
	short		x, y, count;
    long double zr, zi, cr, ci;
    long double rsquared, isquared;


	
    for (y = 0; y < _sizey; y++)
    {
        for (x = 0; x < _sizex; x++)
        {
            zr = 0.0;
            zi = 0.0;
            cr = l + x * (r - l) / _sizex;
			
            ci = t + y * (b - t) / _sizey;
            rsquared = zr * zr;
            isquared = zi * zi;
			
            for (count = 0; ((rsquared + isquared) <= 4.0) && (count < m_Itters); count++)
            {
                zi = zr * zi * 2;
                zi += ci;
				
                zr = rsquared - isquared;
                zr += cr;
				
                rsquared = zr * zr;
                isquared = zi * zi;
            }
			
            if (rsquared + isquared <= 4.0)
				buffer[x][y] = 0;			// printf("*");    
            else
				buffer[x][y] = count;		// printf(" ");    
        }
        printf("\n");
    }
}

void Mandelbrot::Render()
{
	float scale = 0.1f;
	float c;
	glColor3f(1,1,1);
	glBegin(GL_POINTS);
    for (int y = 0; y < 512; y++)
    {
        for (int x = 0; x < 512; x++)
		{
			if(buffer[x][y] > 1)
			{
				c = (buffer[x][y] * (1.0f/m_Itters) );
				glColor3f(1, c, 0);
				/*
				glVertex3f(0.1f*x+scale,	0,	0.1f*y);
				glVertex3f(0.1f*x+scale,	0,	0.1f*y+scale);
				glVertex3f(0.1f*x,			0,	0.1f*y+scale);
				glVertex3f(0.1f*x,			0,	0.1f*y);
				*/
				glVertex2f(x, y);
				
			}
		}
	}
	glEnd();
}

void Mandelbrot::MulItters(float s)
{
	m_Itters *= s;
	cout << "Itterations:" << m_Itters << endl;
}