#include "maths.h"
#include <stdio.h>

mat4f mat4f::operator+ (mat4f param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] + param.cell[i][j];

	return (temp);
}

mat4f mat4f::operator+= (mat4f param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] += param.cell[i][j];
	
	return (temp);
}

mat4f mat4f::operator- (mat4f param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] - param.cell[i][j];

	return (temp);
}

mat4f mat4f::operator-= (mat4f param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] -= param.cell[i][j];
	
	return (temp);
}

mat4f mat4f::operator* (mat4f param)
{
	mat4f temp;

	int i,j,k;
	float sum;

	for (i =0; i<4; ++i){
		for (j=0; j<4; ++j){
			sum = 0; 
			for (k=0; k<4; ++k){
				sum += cell[i][k] * param.cell[k][j];
			}
			temp.cell[i][j] = sum;
		}
	}

	return (temp);
}

mat4f mat4f::operator*= (mat4f param)
{
	mat4f temp;

	int i,j,k;
	float sum;

	for (i =0; i<4; i++){
		for (j=0; j<4; j++){
			sum = 0; 
			for (k=0; k<4; k++){
				sum += cell[i][k] * param.cell[k][j];
			}
			temp.cell[i][j] = sum;
		}
	}

	*this = temp;
	return (temp);
}

mat4f mat4f::operator* (float param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] * param;

	return (temp);
}


mat4f mat4f::operator*= (float param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] *= param;
	
	return (temp);
}

mat4f mat4f::operator/ (float param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] / param;

	return (temp);
}

mat4f mat4f::operator/= (float param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] /= param;
	
	return (temp);
}

mat4f mat4f::operator= (float param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = param;
	
	return (temp);
}

mat4f mat4f::operator= (mat4f param)
{
	mat4f temp;

	int i, j;
	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			temp.cell[i][j] = cell[i][j] = param.cell[i][j];
	
	return (temp);
}

bool mat4f::operator== (mat4f param)
{
	int i,j;

	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			if (fabs(param.cell[i][j] - cell[i][j]) > EQUALENOUGH)
			{
				return 0;
			}

	return 1;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

mat4f::mat4f( void )
{
	for (int i=0; i<4; i++)
		for (int j=0; j<4; j++)
			if (i==j) cell[i][j] = 1.0f;
			else cell[i][j] = 0.0f;
}

mat4f::mat4f( quatf * _param )
{
	*this = QToM4( _param );
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

void mat4f::Set( int column, float n1, float n2, float n3, float n4 )
{
	cell[column][0] = n1;
	cell[column][1] = n2;
	cell[column][2] = n3;
	cell[column][3] = n4;
}

mat4f mat4f::matrix_copy( void )
{
	mat4f temp;

	for( int i = 0; i < 4; i++ )
	{
	    for( int j = 0; j < 4; j++ )
		{
			temp.cell[i][j] = cell[i][j];
		}
	}

	return(temp);
}

mat4f mat4f::matrix_swap_row( mat4f m, int i, int j )
{
	float tmp;
	for( int k = 0; k < 4; k++ )
	{
		tmp = m.cell[i][k];
		m.cell[i][k] = m.cell[j][k];
		m.cell[j][k] = tmp;
	}
	return (m);
}

mat4f mat4f::matrix_scale_row( mat4f m, int i, float s )
{
	int k;
	for( k = 0; k < 4; k++ )
	{
		m.cell[i][k] *= s;
	}
	return (m);
}

mat4f mat4f::Inverse( void )
{
	mat4f T;
	mat4f inv;

	int i, j, k;
	
	for( i = 0; i < 4; i++ )
	{
		for( j = 0; j < 4; j++ )
		{
			inv.cell[i][j] = ( i == j ) ? 1.0f : 0.0f;
		}
	}

	T = matrix_copy();

	for( i = 0; i < 4; i++ )
	{
		if( T.cell[i][i] == 0.0f )
		{
			for( j = i+1; j < 4; j++ )
			{
				if( T.cell[j][i] != 0.0f )
				{
					T = matrix_swap_row( T, i, j );
					inv = matrix_swap_row( inv, i, j );
					break;
				}
			}
			if( j == 4 )
			{
				printf( "Singular trans mat encountered.\n" );
			}
		}
		if( T.cell[i][i] != 1.0f )
		{
			float s = 1.0f / T.cell[i][i];
			T = matrix_scale_row( T, i, s );
			inv = matrix_scale_row( inv, i, s );
		}
		for( j = i+1; j < 4; j++ )
		{
			for( k = i+1; k < 4; k++ )
			{
				T.cell[j][k] -= T.cell[j][i] * T.cell[i][k];
			}
			for( k = 0; k < 4; k++ )
			{
				inv.cell[j][k] -= T.cell[j][i] * inv.cell[i][k];
			}
		}
	}
	for( i = 4-1; i >= 0; i-- )
	{
		for( j = 0; j < i; j++ )
		{
			for( k = 0; k < 4; k++ )
			{
				inv.cell[j][k] -= T.cell[j][i] * inv.cell[i][k];
			}
		}
	}
	return (inv);
}



mat4f mat4f::Transpose( void )
{
	mat4f temp;
	int i,j;

	for ( i=0; i<4; i++)
		for ( j=0; j<4; j++)
			temp.cell[i][j] = cell[j][i];

	return (temp);
}

void mat4f::CreateRotation(vec3f &_around, float angle)
{

	
	float c = (float)cos(angle);
	float s = (float)sin(angle);
	
	this->Set(0, 
		(_around.x * _around.x) * (1.0f - c) + c, 
		(_around.y * _around.x) * (1.0f - c) + (_around.z * s),
		(_around.z * _around.x) * (1.0f - c) - (_around.y * s),
		0.0f);
	
	this->Set(1,
		(_around.x * _around.y) * (1.0f - c) - (_around.z * s),
		(_around.y * _around.y) * (1.0f - c) + c,
		(_around.z * _around.y) * (1.0f - c) + (_around.x * s),
		0.0f);
	
	this->Set(2,
		(_around.x * _around.z) * (1.0f - c) + (_around.y * s),
		(_around.y * _around.z) * (1.0f - c) - (_around.x * s),
		(_around.z * _around.z) * (1.0f - c) + c,
		0.0f);
	
	this->Set(3, 0,0,0,1);

}
