#include "maths.h"

quatf quatf::operator + (quatf param)
{
	quatf temp;

	temp.w = w + param.w;
	temp.x = x + param.x;
	temp.y = y + param.y;
	temp.z = z + param.z;

	return (temp);
}

quatf quatf::operator += (quatf param)
{
	quatf temp;

	temp.w = w += param.w;
	temp.x = x += param.x;
	temp.y = y += param.y;
	temp.z = z += param.z;

	return (temp);
}

quatf quatf::operator - (quatf param)
{
	quatf temp;

	temp.w = w - param.w;
	temp.x = x - param.x;
	temp.y = y - param.y;
	temp.z = z - param.z;

	return (temp);
}

quatf quatf::operator -= (quatf param)
{
	quatf temp;

	temp.w = w -= param.w;
	temp.x = x -= param.x;
	temp.y = y -= param.y;
	temp.z = z -= param.z;

	return (temp);
}

quatf quatf::operator * (quatf param)
{
	quatf temp;

	temp.w = (w * param.w) - (x * param.x) - (y * param.y) - (z * param.z);
	temp.x = (w * param.x) - (x * param.w) - (y * param.z) - (z * param.y);
	temp.y = (w * param.y) - (x * param.z) - (y * param.w) - (z * param.x);
	temp.z = (w * param.z) - (x * param.y) - (y * param.x) - (z * param.w);

	return (temp);
}

quatf quatf::operator *= (quatf param)
{
	quatf temp;

	temp.w = (w * param.w) - (x * param.x) - (y * param.y) - (z * param.z);
	temp.x = (w * param.x) - (x * param.w) - (y * param.z) - (z * param.y);
	temp.y = (w * param.y) - (x * param.z) - (y * param.w) - (z * param.x);
	temp.z = (w * param.z) - (x * param.y) - (y * param.x) - (z * param.w);

	w = temp.w;
	x = temp.x;
	y = temp.y;
	z = temp.z;

	return (temp);
}

quatf quatf::operator * (float param)
{
	quatf temp;

	temp.w = w * param;
	temp.x = x * param;
	temp.y = y * param;
	temp.z = z * param;

	return (temp);
}

quatf quatf::operator *= (float param)
{
	quatf temp;

	temp.w = w *= param;
	temp.x = x *= param;
	temp.y = y *= param;
	temp.z = z *= param;

	return (temp);
}

quatf quatf::operator / (float param)
{
	float fInvScl = 1/param;
	quatf temp;

	temp.w = w * fInvScl;
	temp.x = x * fInvScl;
	temp.y = y * fInvScl;
	temp.z = z * fInvScl;

	return temp;
}

quatf quatf::operator /= (float param)
{
	float fInvScl = 1/param;
	quatf temp;

	temp.w = w *= fInvScl;
	temp.x = x *= fInvScl;
	temp.y = y *= fInvScl;
	temp.z = z *= fInvScl;

	return temp;
}

quatf quatf::operator ~ ( void )
{
	quatf temp;

	temp.w =  w;
	temp.x = -x;
	temp.y = -y;
	temp.z = -z;

	return (temp);
}

//------------------------------------------//
//------------------------------------------//

quatf::quatf( void ) {}

quatf::quatf( float _nw, float _nx, float _ny, float _nz )
{
	w = _nw;
	x = _nx;
	y = _ny;
	z = _nz;
}

quatf::quatf( mat3f * _param )
{
	*this = M3ToQ( _param );
}

quatf::quatf( mat4f * _param )
{
	*this = M4ToQ( _param );
}

//------------------------------------------//
//------------------------------------------//

void quatf::Set(float nw, float nx, float ny, float nz)
{
	w = nw;
	x = nx;
	y = ny;
	z = nz;
}

quatf quatf::Conjugate( void )
{
	quatf temp;

	temp.w =  w;
	temp.x = -x;
	temp.y = -y;
	temp.z = -z;

	return (temp);
}

float quatf::Magnitude( void )
{
	return ((float)sqrt((w * w) + (x * x) + (y * y) + (z * z)));
}

void quatf::Normalize( void )
{
	float fInvMag = 1 / Magnitude();

	if(fInvMag != 1.0f)
	{
		x *= fInvMag;
		y *= fInvMag;
		z *= fInvMag;
		w *= fInvMag;
	}
}

void quatf::RotateBy( quatf param )
{
	quatf tmp(param * (*this) * ~param);
	*this = tmp;
}

float quatf::GetAngle( void )
{
	float temp = 2 * acosf(w);
	return (temp);
}

vec3f quatf::GetAxis( void )
{
	vec3f temp;
	float sa = 1 / sqrtf( 1 - (w * w));
	temp.Set(x * sa, y * sa, z * sa);
	return temp;
}

void quatf::FromAxisAngle( vec3f * rAxis, float fAngle )
{
	rAxis->Normalize();
	float sa = sinf(fAngle * 0.5f);
	float ca = cosf(fAngle * 0.5f);
	x = rAxis->x * sa;
	y = rAxis->y * sa;
	z = rAxis->z * sa;
	w = ca;
}

void quatf::FromEulers(float fX, float fY, float fZ)
{
	float dSY = (float)sin(fZ * 0.5f);
	float dSP = (float)sin(fY * 0.5f);
	float dSR = (float)sin(fX * 0.5f);
	float dCY = (float)cos(fZ * 0.5f);
	float dCP = (float)cos(fY * 0.5f);
	float dCR = (float)cos(fX * 0.5f);

	x = dSR * dCP * dCY - dCR * dSP * dSY;
	y = dCR * dSP * dCY + dSR * dCP * dSY;
	z = dCR * dCP * dSY - dSR * dSP * dCY;
	w = dCR * dCP * dCY + dSR * dSP * dSY;
}

vec3f quatf::GetEulers( void )
{
	vec3f temp;
	float m11 = (w * w) + (x * x) - (y * y) - (z * z);
	float m21 = 2 * (x * y + z * w);
	float m31 = 2 * (z * x - y * w);
	float m32 = 2 * (z * y + x * w);
	float m33 = (w * w) - (x * x) - (y * y) + (z * z);
	temp.Set(((float)atan2(m32, m33)),(asinf(-m31)), ((float)atan2(m21, m11)));

	return temp;
}

quatf quatf::Lerp(quatf _rQuat, float _fInterp)
{
	quatf temp = (((*this) - _rQuat) * _fInterp) + (*this);
	temp.Normalize();
	return temp;
}

quatf quatf::Slerp( quatf _rQuat, float _fInterp )
{
	//Calculate the dot product
	float fDot = ( _rQuat.w * w ) + ( _rQuat.x * x ) + ( _rQuat.y * y ) + ( _rQuat.z * z );

	if(fDot < 0.0f)
	{
		_rQuat -= _rQuat;
		fDot -= fDot;
	}


	if(( fDot < 1.00001f ) && ( fDot > 0.99999f ))
	{
		return Lerp(_rQuat, _fInterp);
	}

	//calculate the angle between the quaternions 
	float fTheta = acosf(fDot);

	return ((*this) * sinf(fTheta * (1 - _fInterp)) + _rQuat * sinf(fTheta * _fInterp))/sinf(fTheta);
}
