#include "maths.h"

vec3f vec3f::operator = (vec3f * _param)
{
	return vec3f(x=_param->x, y=_param->y, z=_param->z);
}

//-------------------------------------------------------
//-------------------------------------------------------

vec3f::vec3f( void )
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}

vec3f::vec3f( float _nx, float _ny, float _nz )
{
	x = _nx;
	y = _ny;
	z = _nz;
}

vec3f::vec3f( vec3f * _param )
{
	x = _param->x;
	y = _param->y;
	z = _param->z;
}

vec3f::vec3f( vec4f * _param )
{
	x = _param->x;
	y = _param->y;
	z = _param->z;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

void vec3f::Set( float nx, float ny, float nz )
{
	x = nx;
	y = ny;
	z = nz;
}

void vec3f::Set( vec3f & _param )
{
	x = _param.x;
	y = _param.y;
	z = _param.z;
}

void vec3f::Set( vec4f & _param )
{
	x = _param.x;
	y = _param.y;
	z = _param.z;
}

float vec3f::Length()
{
	return sqrtf( ( x * x ) + ( y * y ) + ( z * z ) );
}

float vec3f::LengthNoRoot()
{
	return( ( x * x ) + ( y * y ) + ( z * z ) );
}


void vec3f::Scale( float s )
{
	x= x * s;
	y= y * s;
	z= z * s;
}

void vec3f::Normalize()
{
	Scale( 1.0f / Length() );
}

float vec3f::DotProduct( vec3f &v2 )
{
	return( (x * v2.x) + (y * v2.y) + (z * v2.z) );
}

vec3f vec3f::CrossProduct( vec3f &v2 )
{
	vec3f temp;

	temp.x= y * v2.z - z * v2.y;
	temp.y= z * v2.x - x * v2.z;
	temp.z= x * v2.y - y * v2.x;
	return( temp );
}

vec3f vec3f::Lerp( vec3f &v2, float s )
{
	vec3f temp;

	temp.x= x + ( v2.x - x ) * s;
	temp.y= y + ( v2.y - y ) * s;
	temp.z= z + ( v2.z - z ) * s;
	return( temp );
}

float vec3f::SeperationXZ( int nx, int nz )
{
	float dx, dz;

	dx=	x - nx;
	dz=	z - nz;

	return SQRT( SQR( dx ) + SQR( dz ) );
}

vec3f vec3f::Negative( void )
{
	vec3f temp;

	temp.x = -x;
	temp.y = -y;
	temp.z = -z;

	return ( temp );
}

void vec3f::RotateAround(vec3f &_around, float _angle)
{
	mat4f mtx;
	vec4f temp(this);
	
	mtx.CreateRotation(_around, _angle);
	
	temp = temp*mtx;
	
	this->Set(temp.x,temp.y,temp.z);	
}
