#include "maths.h"

vec4f vec4f::operator = (vec4f * _param)
{
	return vec4f(x=_param->x, y=_param->y,z=_param->z, a=_param->a);
}

vec4f vec4f::operator = (vec3f & _param)
{
	return vec4f(x=_param.x, y=_param.y, z=_param.z, a=0);
}

//-------------------------------------------------------
//-------------------------------------------------------

vec4f::vec4f( void )
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
	a = 1.0f;
}

vec4f::vec4f( float _nx, float _ny, float _nz, float _na )
{
	x = _nx;
	y = _ny;
	z = _nz;
	a = _na;
}

vec4f::vec4f( vec3f * _param )
{
	x = _param->x;
	y = _param->y;
	z = _param->z;
	a = 1.0f;
}

vec4f::vec4f( vec4f * _param )
{
	x = _param->x;
	y = _param->y;
	z = _param->z;
	a = _param->a;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

void vec4f::Set( float nx, float ny, float nz, float na )
{
	x = nx;
	y = ny;
	z = nz;
	a = na;
}

// Returns the length of a vector
float vec4f::Length()
{
	return SQRT( SQR( x ) + SQR( y ) + SQR( z ) + SQR( a ) );
}

// Returns the length of a vector
float vec4f::LengthNoRoot()
{
	return( SQR( x ) + SQR( y ) + SQR( z ) + SQR( a ) );
}


//scales the vector by value s
//same as operator *
void vec4f::Scale( float s )
{
	x= x * s;
	y= y * s;
	z= z * s;
	a= a * s;
}

// normalizes the vector to 1 unit length;
void vec4f::Normalize()
{
	Scale( 1.0f / Length() );
}

float vec4f::DotProduct( vec4f &v2 )
{
	return( x * v2.x + y * v2.y + z * v2.z + a * v2.a );
}

vec4f vec4f::CrossProduct( vec4f &v2, vec4f &v3 )
{
    float A, B, C, D, E, F;       // Intermediate Values
	vec4f temp;

    // Calculate intermediate values.

    A = (v2.x * v3.y) - (v2.y * v3.x);
    B = (v2.x * v3.z) - (v2.z * v3.x);
    C = (v2.x * v3.a) - (v2.a * v3.x);
    D = (v2.y * v3.z) - (v2.z * v3.y);
    E = (v2.y * v3.a) - (v2.a * v3.y);
    F = (v2.z * v3.a) - (v2.a * v3.z);

    // Calculate the result-vector components.

    temp.x =   (y * F) - (z * E) + (a * D);
    temp.y = - (x * F) + (z * C) - (a * B);
    temp.z =   (x * E) - (y * C) + (a * A);
    temp.a = - (x * D) + (y * B) - (z * A);

    return temp;
}

vec4f vec4f::Lerp( vec4f &v2, float s )
{
	vec4f temp;
	
	temp.x= x + ( v2.x - x ) * s;
	temp.y= y + ( v2.y - y ) * s;
	temp.z= z + ( v2.z - z ) * s;
	temp.a= a + ( v2.a - a ) * s;
	return( temp );
}

float vec4f::SeperationXZ( int nx, int nz )
{
	float dx, dz;
	
	dx=	x - nx;
	dz=	z - nz;
	
	return SQRT( SQR( dx ) + SQR( dz ) );
}

vec4f vec4f::Negative( void )
{
	vec4f temp;
	
	temp.x = -x;
	temp.y = -y;
	temp.z = -z;
	temp.a = -a;
	
	return ( temp );
}

vec4f vec4f::operator*(mat4f &_param)
{
	vec4f a,b,c,d,temp;

	a.Set(_param.cell[0][0],_param.cell[0][1],_param.cell[0][2],_param.cell[0][3]);
	b.Set(_param.cell[1][0],_param.cell[1][1],_param.cell[1][2],_param.cell[1][3]);
	c.Set(_param.cell[2][0],_param.cell[2][1],_param.cell[2][2],_param.cell[2][3]);
	d.Set(_param.cell[3][0],_param.cell[3][1],_param.cell[3][2],_param.cell[3][3]);

	a*=this->x;
	b*=this->y;
	c*=this->z;
	d*=this->a;

	temp.x = a.x + b.x + c.x + d.x;
	temp.y = a.y + b.y + c.y + d.y;
	temp.z = a.z + b.z + c.z + d.z;
	temp.a = a.a + b.a + c.a + d.a;

	return temp;
}